struc header
  next: resd 1
  attr: resw 1
  strat: resw 1
  intr: resw 1
  name: resb 8
endstruc

struc drivereq
  .len: resb 1
  .unit: resb 1
  .cmd: resb 1
  .status: resw 1
  .dosq: resd 1
  .devq: resd 1
endstruc

struc initreq
  .hdr: resb drivereq_size
  .numunits: resb 1
  .brkaddr: resd 1
  .bpbaddr: resd 1
endstruc

; Status return bits- high
%define STATUS_ERROR      (1 << 15)
%define STATUS_BUSY       (1 << 9)
%define STATUS_DONE       (1 << 8)

; Error codes (Status return bits- low)
%define WRITE_PROTECT     0
%define UNKNOWN_UNIT      1
%define DRIVE_NOT_READY   2
%define UNKNOWN_COMMAND   3
%define CRC_ERROR         4
%define BAD_DRIVE_REQ     5
%define SEEK_ERROR        6
%define UNKNOWN_MEDIA     7
%define SECTOR_NOT_FOUND  8
%define OUT_OF_PAPER      9
%define WRITE_FAULT     0xA
%define READ_FAULT      0xB
%define GENERAL_FAILURE 0xC

hdr:
istruc header
  at next, dd -1
  at attr, dw 0x0000
  at strat, dw strategy
  at intr, dw interrupt
  at name, db 'SET66Hxx'
iend

; Driver data
packet_ptr dd 0

strategy:
  mov cs:[packet_ptr], bx
  mov cs:[packet_ptr+2], es
  retf

; The available source for MS-DOS don't preserve flags in drivers, and no point
; in preserving SP.
interrupt:
  push ax
  push cx
  push dx
  push bx
  push si
  push di
  push bp
  push ds
  push es

  les di, cs:[packet_ptr]
  mov si, es:[di + drivereq.cmd]
  cmp si, 0
  ja .bad_cmd

  shl si, 1
  jmp init

.bad_cmd:
  mov al, UNKNOWN_COMMAND
.err:
  xor ah, ah
  or ah, (STATUS_ERROR | STATUS_DONE) >> 8
  mov es:[di + drivereq.status], ax
  jmp interrupt.end

.busy:
  mov word es:[di + drivereq.status], STATUS_DONE | STATUS_BUSY
  jmp interrupt.end

.exit:
  mov word es:[di + drivereq.status], STATUS_DONE
.end:
  pop es
  pop ds
  pop bp
  pop di
  pop si
  pop bx
  pop dx
  pop cx
  pop ax
  retf

; Init data does not need to be kept, so it goes last.
res_end:
init:
  call MAIN
  push cs
  pop ds
  mov dx, install_msg
  mov ah, 0x09
  int 0x21
  mov word es:[di + initreq.brkaddr], res_end
  mov word es:[di + initreq.brkaddr + 2], cs
  jmp interrupt.exit

install_msg db 'SET66h Complete.', 0xD, 0xA, '$'









setfeature66:
	mov	dx,642h
	mov	al,66h
	out	dx,al
	mov	cx,2000h
wait0:
	out	5fh,al
	out	5fh,al
	mov	dx,64eh
	in	al,dx
	cmp	al,50h
	je	waitnuke
	loop	wait0
waitnuke:
	mov	al,0efh
	out	dx,al
	out	5fh,al
	ret


END:
	ret
	
	
MAIN:
	mov	dx,432h
	mov	al,1
	out	dx,al		;ZJ_
	out	5fh,al

	out	5fh,al
	out	5fh,al
	mov	dx,64ch
	mov	al,10h
	out	dx,al		;X[u
	out	5fh,al
	mov	dx,64ch
	in	al,dx
	and	al,1fh		;0a0htŋAĂ@킪
	cmp	al,10h
	jne	SecondaryMaster
	call	setfeature66

SecondaryMaster:
	out	5fh,al
	out	5fh,al

	mov	dx,64ch
	mov	al,0h
	out	dx,al		;}X^
	out	5fh,al
	in	al,dx
	and	al,1fh
	cmp	al,0
	jne	PrimarySlave
	call	setfeature66

PrimarySlave:
	mov	dx,432h
	mov	al,0
	out	dx,al		;vC}

	out	5fh,al
	out	5fh,al

	mov	dx,64ch
	mov	al,10h
	out	dx,al		;X[u
	out	5fh,al
	in	al,dx
	and	al,1fh
	cmp	al,10h		; hc ЂǂoO
	jne	PrimaryMaster
	call	setfeature66

PrimaryMaster:
	mov	dx,64ch
	mov	al,0h
	out	dx,al		;}X^
	out	5fh,al
	in	al,dx
	and	al,1fh
	cmp	al,0
	jne	END
	call	setfeature66
	jmp	END

